@@ -1,20 +1,17 @@
-# Copyright (c) 2015-2016, 2018 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2016 Ceridwen <ceridwenv@gmail.com>
-# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2021 David Liu <david@cs.toronto.edu>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
+from __future__ import annotations
 
 import collections
-from functools import lru_cache
+from typing import TYPE_CHECKING
 
 from astroid.context import _invalidate_cache
 
+if TYPE_CHECKING:
+    from astroid import NodeNG
+
 
 class TransformVisitor:
     """A visitor for handling transforms.
@@ -23,22 +20,18 @@ class TransformVisitor:
     :meth:`~visit` with an *astroid* module and the class
     will take care of the rest, walking the tree and running the
     transforms for each encountered node.
-    """
 
-    TRANSFORM_MAX_CACHE_SIZE = 10000
+    Based on its usage in AstroidManager.brain, it should not be reinstantiated.
+    """
 
     def __init__(self):
         self.transforms = collections.defaultdict(list)
 
-    @lru_cache(maxsize=TRANSFORM_MAX_CACHE_SIZE)
-    def _transform(self, node):
+    def _transform(self, node: NodeNG) -> NodeNG:
         """Call matching transforms for the given node if any and return the
         transformed node.
         """
         cls = node.__class__
-        if cls not in self.transforms:
-            # no transform registered for this class of node
-            return node
 
         transforms = self.transforms[cls]
         for transform_func, predicate in transforms:
@@ -73,7 +66,7 @@ class TransformVisitor:
 
         return self._visit(node)
 
-    def register_transform(self, node_class, transform, predicate=None):
+    def register_transform(self, node_class, transform, predicate=None) -> None:
         """Register `transform(node)` function to be applied on the given
         astroid's `node_class` if `predicate` is None or returns true
         when called with the node as argument.
@@ -83,7 +76,7 @@ class TransformVisitor:
         """
         self.transforms[node_class].append((transform, predicate))
 
-    def unregister_transform(self, node_class, transform, predicate=None):
+    def unregister_transform(self, node_class, transform, predicate=None) -> None:
         """Unregister the given transform."""
         self.transforms[node_class].remove((transform, predicate))
 
